var ioclient = require('socket.io-client');
var memoryDb = require('./memoryDb.js');
var request = require('request');
var handler = require('./event_handler').getHandler();
var config = require('../config/conftool').getConf();
var cloudinitiated;

exports.boot = function (){

  console.log('booting cloud connection...', cloudinitiated);
  //Simplified memoryDb call 
  memoryDb.checkCloudInit(function(cloudInfo){
    console.log("Checked cloud init", cloudInfo);
    if(cloudInfo){
      console.log('cloud initiated'); 
      cloudinitiated = cloudInfo;
      connectToCloud();
    }
  });
  
  handler.on('cloudinitiated', function(){
    connectToCloud();
  });

  handler.on('cloud_save', function(data){
    if(cloudinitiated){
      console.log('saving to cloud');
      saveToCloud(data); 
    }else{
      if(cloudinitiated){
        console.log('saving to cloud');
        saveToCloud(data); 
      }else{
        console.log('cloud not initiated not saving to cloud');
      }
    }
  });

  function connectToCloud(){
    var disconnectTimer = null;
    console.log('requesting a socket key from cloud', cloudinitiated);
    var oauth = { consumer_key : config.cc.consumerKey,
                  consumer_secret: config.cc.consumerSecret,
                  token: cloudinitiated.token.token,
                  token_secret: cloudinitiated.token.attributes.tokenSecret
                };
    var getKeyAndConnect = function(){
      var gw = memoryDb.statusDb.get('gateway');
      if(gw){
        console.log("Using gatewayId :" + gw);
      }else{
        console.log("No gatewayId yet");
      }
      request.post({url: 'http://interact.gridreact.com/api/connection_key',
               oauth: oauth, body: {gatewayId: gw},  json : true}
               ,function(error, response, body ){
          if(body.gatewayId && !gw){
            memoryDb.statusDb.set('gateway', body.gatewayId);
          }
          if(!error && response.statusCode !== 500){
          console.log('received socket key', body);
          var connect = function(){
            socket = ioclient.connect('http://connector.gridreact.com',
                                     {
                                      transports: ['websocket'],
                                      'connect timeout' : 5000, 
                                      'max reconnection limit': Infinity});
          }
          connect();
          socket.on('connect_failed', function(){
            console.log('connection to the cloud cordinator failed, trying again later');
            disconnectTimer = setTimeout(function(){
              console.log('attempting to connect');
              connect();
              }, 30000);
          });
          socket.on('authsuccess', function(data){
            console.log("sucessful auth",  data);
          });
          socket.on('authfailure', function(data){
            console.log("bad auth tryin again");
          });
          socket.on('connect', function(){
            if(disconnectTimer)
              clearInterval(disconnectTimer);
            socket.emit('auth', body);
            handler.emit('web_up');
            console.log('connected to cloud cordinator, ready for actions');
          });
          socket.on('action', function(data){
            console.log('received action from cloud cordinator', data);
            handler.emit('web_action', data);
          });
          socket.on('disconnect', function(){
            handler.emit('web_down');
            console.log('disconnected from cloud cordinator, will try to reconnect soon');
            disconnectTimer = setTimeout(function(){
              console.log('attempting to connect');
              connect();
              }, 30000);
          });
        }else{

          disconnectTimer = setTimeout(function(){
            console.log('attempting to connect');
            getKeyAndConnect();
            }, 5000);
          console.log('did not connect waiting to try again');
          console.log('error', error, response.body.error);
        }
      });
    }
    getKeyAndConnect();

  }

  function saveToCloud(data){
    console.log(data);
    var oauth = { consumer_key : config.cc.consumerKey,
                  consumer_secret: config.cc.consumerSecret,
                  token: cloudinitiated.token.token,
                  token_secret: cloudinitiated.token.attributes.tokenSecret
                };
    console.log(oauth);
    request.put({url: 'http://interact.gridreact.com/api/sensor/' + data.sensorId,
             oauth: oauth, json : true, body: data}
             ,function(error, response, body ){
      if(!error){
        console.log(body);
      }else{
        console.log('error', error);
      }
    });

  }
}
