var util = require('util');
var gpio = require('onoff').Gpio;
var GPIO_MAC = "gpio-driver-gw";

var outputs = [4, 17, 23, 24, 25];

var output_pins = {};
outputs.forEach(function(pin){
  output_pins[pin] = new gpio(pin, 'out');
});

module.exports.boot = function(emitter) {
	var handler = emitter;
  
  var doAction = function(action){
    switch(action.name){
      case 'pullup':
        pullup(action);
      break;
      case 'pulldown':
        pulldown(action);
      break;
      default:
        console.log("Action " , action.name , " Not supported");
      break;
    }
  }

  var pullup = function(action){
    if( output_pins[action.values.pin]){
      output_pins[action.values.pin].writeSync(1);
    }else{
      console.log("Pin does not exist", action.values.pin);
    }
  }

  var pulldown = function(action){
   if( output_pins[action.values.pin]){
      output_pins[action.values.pin].writeSync(0);
    }else{
      console.log("Pin does not exist", action.values.pin);
    }
  }

  handler.on(GPIO_MAC, function(action){
    console.log("GPIO Driver got a request");
    try{
      doAction(action);  
    }catch(e){
      console.log("Problem executing action", action);
      console.log(e);
    }
  });

  setTimeout(function(){ 
    handler.emit('sensor_reading', {
        sensorId: GPIO_MAC,
        payload:  {type: 300}
    });
    console.log('Registered GPIO Driver');
  }, 5000);

}

