var util = require('util');
var XBee = require('svd-xbee').XBee;
var JParser = require('jParser');
var jsonPacker = require('./xbee_libs/jsonPacker.js');
var _ = require('underscore');

var xbee = new XBee({
	port: '/dev/ttyUSB0',
	baudrate: 9600,
  config: {},
  emit_buffers :true
});

var packet_patterns = require('../sensorData');

module.exports.boot = function(emitter) {
	var handler = emitter;

	xbee.on("initialized", function(config) {
		console.log("XBee initilized: %s", util.inspect(config));
    xbee.discover();
  });

	xbee.on("newNodeDiscovered", function(node) {
	 // handler.emit('sensor_annouce', {id: node.remote64.hex, status: 'on', node: node});	
	  if(handler.listeners(node.remote64.hex).length === 0){
      handler.on(node.remote64.hex, function(action){
        console.log('creating binary packet from action');
        var struct = _.find(packet_patterns[action.type].actions, function(a){
                        return a.name == action.name;});
        var jp = new jsonPacker(action.values, struct, {size:100});
        try{ 
          var packed = jp.pack();
          node.send(packed);
        }catch(e){
          console.log(e.stack);
          console.log(action);
          console.log('error packing');
        }
      });
    }else{
      console.log('sensor registered but already handling requests to that sensor');
    }

    node.on("data", function(text , packet) {
      var data = new Buffer(packet.rawData);
      console.log('Got a packet from a sensor', data);
			//Every packet that comes in should be parsed based on its sensor type.
      //sensor type is defined by the first byte of the array.
      //0x00 is a thermostat
      var type = data[0];
      console.log('sensor with type ' + type + ' sent data');
      if(packet_patterns[type]){
        var parser = new jParser(data, {pattern : packet_patterns[type].reading});
        var packet = parser.parse('pattern');
        console.log(packet);
        handler.emit('sensor_reading', {
             sensorId: node.remote64.hex,
             payload:  packet
        }); 
      }else{
        console.log('unsupported sensor type');
      }
	});

  });
  xbee.on('error', function(err){
    console.log(err);
  });
  try{ 
	xbee.init(function(){
    console.log('created serial connection to xbee');
  });
  }catch(e){
    console.log(e, 'error initializing xbee cordinating... waiting..');
  }
}

